# Generated by roxygen2: do not edit by hand

export(deltamethod)
export(draw_key_hpath)
export(draw_key_pointrangeh)
export(expand_modelframe)
export(forest_plot)
export(get_sample_data)
export(run_interactiveforestplot)
import(data.table, except = c(last,between,first))
importFrom(colourpicker,colourInput)
importFrom(stats,deriv)
